



#include <CompuCell3D/Simulator.h>
#include <CompuCell3D/Potts3D/Potts3D.h>
#include <CompuCell3D/Field3D/Field3D.h>
#include <CompuCell3D/Field3D/WatchableField3D.h>
#include <CompuCell3D/Boundary/BoundaryStrategy.h>

#include <CompuCell3D/Potts3D/CellInventory.h>
#include <CompuCell3D/Automaton/Automaton.h>
#include <BasicUtils/BasicString.h>
#include <BasicUtils/BasicException.h>
#include <PublicUtilities/StringUtils.h>
#include <algorithm>

#include <CompuCell3D/plugins/PixelTracker/PixelTrackerPlugin.h>

using namespace CompuCell3D;


#include <iostream>
using namespace std;

#include "AverageConcentration.h"

AverageConcentration::AverageConcentration() : cellFieldG(0),sim(0),potts(0),xmlData(0),boundaryStrategy(0),automaton(0),cellInventoryPtr(0){}

AverageConcentration::~AverageConcentration() {
}


void AverageConcentration::init(Simulator *simulator, CC3DXMLElement *_xmlData) {
  xmlData=_xmlData;
  
  potts = simulator->getPotts();
  cellInventoryPtr=& potts->getCellInventory();
  sim=simulator;
  cellFieldG = (WatchableField3D<CellG *> *)potts->getCellFieldG();
  fieldDim=cellFieldG->getDim();


  simulator->registerSteerableObject(this);

  update(_xmlData,true);

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void AverageConcentration::extraInit(Simulator *simulator){
    //PUT YOUR CODE HERE
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AverageConcentration::start(){

  //PUT YOUR CODE HERE
	CellInventory::cellInventoryIterator cInvItr;
	CellG * cell;
	

	cerr<<"INSIDE START FUNCTION"<<endl;
	for(cInvItr=cellInventoryPtr->cellInventoryBegin() ; cInvItr !=cellInventoryPtr->cellInventoryEnd() ;++cInvItr )
	{
		cell=cellInventoryPtr->getCell(cInvItr);
		cell->targetVolume=25.0;
		cell->lambdaVolume=2.0;
		cerr<<"cell->targetVolume="<<cell->targetVolume<<endl;
	}
	

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void AverageConcentration::step(const unsigned int currentStep){
    //REPLACE SAMPLE CODE BELOW WITH YOUR OWN
	CellInventory::cellInventoryIterator cInvItr;
	CellG * cell;


	cerr<<"currentStep"<<currentStep<<endl;
	Field3DImpl<float>* field= sim->getConcentrationFieldByName("FGF");

	bool pluginAlreadyRegisteredFlag;
	pixelTrackerPlugin=(PixelTrackerPlugin *)Simulator::pluginManager.get("PixelTracker",&pluginAlreadyRegisteredFlag); //this will load VolumeTracker plugin if it is not already loaded
	BasicClassAccessor<PixelTracker> *accessor=pixelTrackerPlugin->getPixelTrackerAccessorPtr();


    //cerr<<"currentStep="<<currentStep<<endl;



	for(cInvItr=cellInventoryPtr->cellInventoryBegin() ; cInvItr !=cellInventoryPtr->cellInventoryEnd() ;++cInvItr )
	{
		cell=cellInventoryPtr->getCell(cInvItr);
		set<PixelTrackerData > & pixelSetRef=accessor->get(cell->extraAttribPtr)->pixelSet;
		double concentration=0.0;
		for (set<PixelTrackerData >::iterator sitr = pixelSetRef.begin() ; sitr!=pixelSetRef.end();++sitr){
			concentration+=field->get(sitr->pixel);
		}
		concentration/=pixelSetRef.size();
		//cerr<<"concentration="<<concentration<<endl;

        //cerr<<"cell.id="<<cell->id<<" vol="<<cell->volume<<endl;
    }

}


void AverageConcentration::update(CC3DXMLElement *_xmlData, bool _fullInitFlag){

    //PARSE XML IN THIS FUNCTION
    //For more information on XML parser function please see CC3D code or lookup XML utils API
    automaton = potts->getAutomaton();
    ASSERT_OR_THROW("CELL TYPE PLUGIN WAS NOT PROPERLY INITIALIZED YET. MAKE SURE THIS IS THE FIRST PLUGIN THAT YOU SET", automaton)
   set<unsigned char> cellTypesSet;

    CC3DXMLElement * exampleXMLElem=_xmlData->getFirstElement("Example");
    if (exampleXMLElem){
        double param=exampleXMLElem->getDouble();
        cerr<<"param="<<param<<endl;
        if(exampleXMLElem->findAttribute("Type")){
            std::string attrib=exampleXMLElem->getAttribute("Type");
            // double attrib=exampleXMLElem->getAttributeAsDouble("Type"); //in case attribute is of type double
            cerr<<"attrib="<<attrib<<endl;
        }
    }
    
    //boundaryStrategy has information aobut pixel neighbors 
    boundaryStrategy=BoundaryStrategy::getInstance();

}

std::string AverageConcentration::toString(){
   return "AverageConcentration";
}

std::string AverageConcentration::steerableName(){
   return toString();
}
        
